#!/usr/bin/env python 



# This acts just like "#include" in C
execfile("../../../tools/build_library.py")



#**********************************************************************************************
# Customization Section
#**********************************************************************************************


# Usage and notes:
#
#  1) Edit this file to change build parameters such as "par" switches, etc. "Implement.py"
#     customizes the way the actual script 'build_libray.py' runs ( see 'execfile' above ).
#
#  2) To build on the local machine, execute this script directly:
#
#             linux_prompt>implement.py
#
#     Important: When building locally the (Xilinx) tools versions are not fixed, but depend
#     on the user's login settings.
#
#  3) Normally, build on the server farm with
#
#             linux_prompt> lsf_build
#
#     The two advantages are 1) The build completes much faster.
#                            2) The script 'lsf_build' dictates the Xilinx tool version used. 
#
#
#  4) The script is set up to run bitsweeper, gather a copy of all rtl modules for backup, run
#     multiple build attempts until a minimum timing score that you specify is met, and publish
#     the results to a shared drive for distribution. It also can be run to attempt to duplicate
#     a previously built image in order to prove that the source was archived in source control
#     properly.
#
#  5) When ready to build and publish an official revision, edit your code to increment REVID
#     by 1. Numbers are not normally skipped, but can be as long as the REVID always increases.
#
#
#  6) To build to a local subfolder only and not write results automatically to the shared drive,
#     change the REVID in your rtl code to 0. This is for building experimental images to test in the
#     lab before general distribution. Any zero REVID read back by S/W is understood to be an
#     unofficial image, and so this is a safety measure to make sure only official revisions are
#     distributed. When the build script completes, manually "cp -r" the output folder structure to a
#     temporary folder under '/h/hss-series/hardware/' (By convention mapped to F: in the PC world).
#     Rename the output folder to include the FPGA name, appID, latest official revision on which
#     it is based, and some description of the change. For example,
#     'F3_Base_AppID_0x2_Revid_0x4_PCIe_Test_Points.'
#
#  7) A second way to build locally only is to set the 'Min_Score' parameter to a negative number. 
#     This is an impossible timing score to meet, so the script will not publish the results.

#  8) To see the full printout from a tool such as "map" or "par", add it to 'verbose_commands[]'
#     below. Add the keyword "all" to show live output for all tools.
#
#  9) Each tool invoked by this script is run after cd into a new subfolder that is created to
#     contain all the results of the build. Running from within helps ensure that the
#     subfolder contains all the needed input files and log file outputs. If the build process 
#     compeletes fully, the subfolder is copied to the shared drive.
#
# 10) The subfolder structure is as follows:
#          rev_NNNN_NNNN
#               Export
#               src
#               bitsw
#               par
#
#     The FPGA loadable image, bitsweeper output files, and certain par summary information is copied
#     to 'Export' for use by the driver and tools. 'src' contains a snapshot of all the Verilog source
#     files in one flat directory. It is not only a backup, but often helpful for examining differences
#     between any two builds. 'bitsw', 'syn', and 'par' are the working folders for their respective
#     tools.
#
# 11) Regardless of whether the screen output is viewed live, all command outputs are saved in the
#     'command.log' files within the subfolder structure.
#
#
# 12) If the rtl contains a NOP_VALUE constant, the script updates it with a new random value 
#     and makes a completly new build attempt if the required minimum timing score is not met.
#     If you don't use a NOP_VALUE, set NOP_VALUE = "" below.
#
#     A design can often be solvable and constrained correctly, yet not meet timing in given par
#     attempts. This is because place and route tools only explore the solution space near the initial
#     conditions before giving up. A slight source code change, even when unrelated to difficult
#     timing paths, alters the initial conditions effectively.
#
#     Note that the design must implement NOP_VALUE in a way that a) is not synthesized out and b) changes
#     the design with no real functional differences.
#
# 13) The script does not interact with the source control repository, except to verify a match
#     between the repository path that your sandbox points to and the would-be shared drive
#     path for publishing. This check helps ensure that when you copy an old application to
#     a new application that you don't publish to the shared folder of the old application.
#     In other words, you forgot to change "APP_ID" in your rtl.
#
# 14) The script checks if REVID has already been published to the shared drive. If so,
#     it compiles locally and reports if the new .hsi binary file matches the published
#     build. If it does, the script prompts if you want to move the original aside and
#     replace it.
#     
# 15) The build-and-compare is provided in order to verify that the source of an existing image
#     has been properly saved. By convention we "tag" all official images after they are published.
#     In svn this means copying the HEAD revision of "trunk" to new folder under "/tags".
#
#     Procedure:
#
#     a) Take out a fresh sandbox not from /trunk, but from /tag/revid for the REVID you are
#        trying to duplicate.
#
#     b) Run this script as you normally would and answer 'n' when prompted to replace the
#        original, or invoke with  ./lsf_build --pass-fail
#
#     c) When the script finishes it tells you whether you succeeded or not in replicating
#        If successful, say "no" when asked about replacing the published folder.
#
#
#     d) If not successful, use Beyond Compare to compare the new "src" snapshot folder to the
#        one originally published to make sure all the RTL files match. If the source was not
#        committed correctly or tagged incorrectly originally, change the sandbox source files
#        to match the published files ( in the original source locations, not the snapshot 'src'
#        directory ). Delete the local build results and repeat the build. When a binary image
#        is finally successfully matched, commit the changes back to the tag  in subversion.
#        Normally, tags are not altered but it's more important that tags match the published
#        images.
#
#

FPGA_NAME = "G1"
REVID = "G1_REVID"
APP_ID = "G1_APPID"
CUSTOMER_ID = "G1_CUSTOMERID"
NOP_VALUE = "RO_G1_NOP_VALUE" # Set this to "" if NOP_VALUE is not used.

BITSWEEPER_FPGA_NAME = "G1"
NOP_VALUE_FILE = "../src/g1_core_ctrl_stat.v"



# Indicate which rtl source file contains the ID constants above
ID_FILE = "../src/g1_core_ctrl_stat.v"

# Determine where the script publishes official revisions
Shared_Drive_Template = "h/hss-series/hardware/builds/fioxi_instrument/application_%04X/g1/"


vtop = "g1_top"
prjtop = "implement.tcl"


# For ISE, the .prj file listed all the source files and was used to create a source snapshot.
# For vivado, you don't have to name all the individual rtl and .xdc files that make up your IP cores.
# Therefore we invoke 'get_files -compile_order' in 'implement.tcl' to create the file named
# below that lists all the source files. Taking a snapshot now occurs after par instead of before it.

snapshot_list_fname = "g1_top_snapshot_list.rpt"


# These commands are run from within ./<subfolder>/src
build_preclean = '''

# Before doing anything, remove any output files from build folder that
# are version controlled.

rm -f ../../g1_top.pad
rm -f ../../par_timing_summary.txt
rm -f ../../bitsw.dat
#rm -f ../../bitsw.xml

# Remove other possible leftover files from an old build.

rm -f ../../g1_top.bit
rm -f ../../g1_top.ngc



# Save a copy of the build scripts
cp ../../implement.tcl .
cp ../../implement.py .
cp ../../lsf_build .




'''



# Each command in par_commands is run from within ./<subfolder>/par
par_commands = '''

# Show the tools versions
which vivado

# Copy or move place and route inputs into the working folder

cp ../../g1_pim0_rs485.xdc .
cp ../../g1_pim1_hotlinkecl.xdc .
cp ../../g1_top.xdc .
cp ../../implement.tcl .




##########################################################################################
# G1 synth, place and route tool chain:
#
# input files => (tool) => output files + {log or report files}
# 
# -- Generate runtime image
#
# g1 HDL + g1_top.xdc + g1_pim0.xdc + g1_pim1.xdc => (synth_design) => g1_top_post_synth + 
#                                           {g1_top_post_synth_util.rpt + 
#                                           g1_top_post_synth_timing_summary.rpt + 
#                                           g1_top_post_synth_power.rpt}
#
# g1_top_post_synth => (opt_design + power_opt_design + place_design + phys_opt_design)
#                       => g1_top_post_place + {g1_top_post_synth_util.rpt + 
#                                               g1_top_post_place_timing_summary.rpt}
#
# g1_top_post_place => (route_design) => g1_top_post_route + {g1_top_post_route_timing_summary.rpt + 
#                                 g1_top_post_route_timing.rpt + g1_top_clock_util.rpt + 
#                                 g1_top_post_route_util.rpt + g1_top_post_route_power.rpt + 
#                                 g1_top.pad}
#
# g1_top_post_route => (write_bitstream) => g1_top.bin
#
# -- Timing analysis(See route_design output) and post-route simulation models --
#
# g1_top_post_route => (write_verilog) => g1_top_impl_netlist.v
# 
#########################################################################################
#
# Note: 1) Vivado will create a .bin file (*) that we rename .hsi. 
#
# (*) A .bin file is a .bit file without the Xilinx header
#
#########################################################################################

vivado -mode batch -source implement.tcl


# Copy the .pad files up to the /build folder for publishing with the revision.

cp g1_top.pad ../../
cp g1_top_pad.csv ../../

# Move the bitstream files up to the /build folder for publishing the revision.

mv g1_top.bit ../../
mv g1_top.bin ../../



# Copy/rename some build output files up to the /build folder for revision control.
# These are for comparison with the last revid. Strip out time/date/host so that
# only real changes will show as svn (M) modified files. These files dont usually
# change and new differences can reveal build problems.

egrep -v '(Date|Host)' g1_top.pad                          > ../../current_pad_file.rpt
egrep -v '(Date|Host)' g1_top_syn_constraint_order.rpt     > ../../current_syn_constraint_order.rpt
egrep -v '(Date|Host)' g1_top_imp_constraint_order.rpt     > ../../current_imp_constraint_order.rpt
egrep -v '(Date|Host)' g1_top_syn_rtl_source_order.rpt     > ../../current_rtl_source_order.rpt
egrep -v '(Date|Host)' g1_top_post_imp_drc.rpt             > ../../current_post_imp_drc.rpt
egrep -v '(Date|Host)' g1_top_post_route_timing_errors.rpt > ../../current_post_route_timing_errors.rpt
egrep -v '(Date|Host)' g1_top_invalid_constraints.xdc      > ../../current_invalid_constraints.rpt





# Create a readme file for the build.sh, and automatically write
# with the Vivado tools version.

echo "This design was last created with the following Xilinx Vivado tools version:" > ../../build.sh.readme
echo " " >> ../../build.sh.readme
grep -i '# Vivado v' vivado.log >> ../../build.sh.readme



'''


# Each command in export_commands is run from within ./<subfolder>/Export
export_commands = '''





# Show the tools versions
which vivado

# Copy or move the files to the /Export folder 
cp ../../Feature_Notes.txt .
cp ../../Algorithms.txt .
cp ../../bitsw.h ./g1.h
cp ../../bitsw.xml ./g1.xml
cp ../../par_timing_summary.txt .
cp ../../map_utilization_summary.txt .
cp ../../g1_top.pad .
cp ../../g1_top_pad.csv .
mv ../../*.bit .
mv ../../*.hsi .
# Keep only the renamed .bit file.
rm ./g1_top.bit

'''


# If you want live printout instead of dots sent to screen, add the name of the command here.
# Add "all" if you want everything printed.
verbose_commands = ["which"]




#
# mrp_filename and print_utilization_terms are for printing 'map_utilization_summary.txt'
# The build script will print any line from the mrp file that contains one of the strings
# in 'print_utilization_terms'.
#

mrp_filename = "g1_top_post_route_util.rpt"

print_utilization_terms = '''

Site Type
Slice
Block RAM Tile
MMCME2_ADV
PLLE2_ADV
PCIE_2_1
GTXE2_CHANNEL
BUFGCTRL
'''




# Repeat PAR until you get a good Score. Only matters if you use NOP_VALUE

               

# Maximum number of par attempts allowed if NOP_VALUE is used.
Max_Loops = 10

#################################################################################################
# Min_Score
#################################################################################################
#
# 0 or Positive number:
#
#      1) Finish and write to shared area when timing score is less than or equal to this value.
#      2) If not and NOP_VALUE is used, try again with a new NOP_VALUE until Min_Score is met or
#         until Max_Loops is met.
#
# Negative number:
#
#      1) Never publish to shared drive. 
#      2) If NOP_VALUE is used, keep repeating build process with a different NOP_VALUE each time.
#         Quit when Max_Loops is met.


               
Min_Score = 0





# Make the random NOP_VALUES repeatable by using a fixed seed.
#NOP_SEED = 0x1
NOP_SEED = None




# Set to true if "build.sh" script is desired. Used for customer example apps only.
create_build_dot_sh = True



#**********************************************************************************************
# End of Customization Section
#**********************************************************************************************


# Settings are all set, so do the build.
do_build(cpld = False)

#**********************************************************************************************
# End End End End End End End End End End End End End End End End End End End End End End End 
#**********************************************************************************************


